@@ -94,7 +94,24 @@ func TestClone(t *testing.T) {
 
 	password, _ := server.URL.User.Password()
 	namespace := "test"
+	defaultSizeGiB := int32(5)
 	vm := simulator.Map.Any("VirtualMachine").(*simulator.VirtualMachine)
+
+	machine := object.NewVirtualMachine(session.Client.Client, vm.Reference())
+	devices, err := machine.Device(context.TODO())
+	if err != nil {
+		t.Fatalf("Failed to obtain vm devices: %w", err)
+	}
+	disks := devices.SelectByType((*types.VirtualDisk)(nil))
+	if len(disks) < 1 {
+		t.Fatal("Unable to find attached disk for resize")
+	}
+	disk := disks[0].(*types.VirtualDisk)
+	disk.CapacityInKB = int64(defaultSizeGiB) * 1024 * 1024 // GiB
+	if err := machine.EditDevice(context.TODO(), disk); err != nil {
+		t.Fatalf("Can't resize disk for specified size")
+	}
+
 	credentialsSecret := corev1.Secret{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:      "test",
@@ -106,6 +123,17 @@ func TestClone(t *testing.T) {
 		},
 	}
 
+	userDataSecretName := "vsphere-ignition"
+	userDataSecret := corev1.Secret{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      userDataSecretName,
+			Namespace: namespace,
+		},
+		Data: map[string][]byte{
+			userDataSecretKey: []byte("{}"),
+		},
+	}
+
 	testCases := []struct {
 		testCase              string
 		cloneVM               bool
@@ -123,6 +151,11 @@ func TestClone(t *testing.T) {
 				Workspace: &vsphereapi.Workspace{
 					Server: server.URL.Host,
 				},
+				DiskGiB:  defaultSizeGiB,
+				Template: vm.Name,
+				UserDataSecret: &corev1.LocalObjectReference{
+					Name: userDataSecretName,
+				},
 			},
 			cloneVM:     true,
 			machineName: "test0",
@@ -137,10 +170,50 @@ func TestClone(t *testing.T) {
 					Server: server.URL.Host,
 					Folder: "custom-folder",
 				},
+				DiskGiB:  defaultSizeGiB,
+				Template: vm.Name,
+				UserDataSecret: &corev1.LocalObjectReference{
+					Name: userDataSecretName,
+				},
 			},
 			cloneVM:     true,
 			machineName: "test1",
 		},
+		{
+			testCase: "clone machine and increase disk",
+			providerSpec: vsphereapi.VSphereMachineProviderSpec{
+				CredentialsSecret: &corev1.LocalObjectReference{
+					Name: "test",
+				},
+				Workspace: &vsphereapi.Workspace{
+					Server: server.URL.Host,
+				},
+				DiskGiB:  defaultSizeGiB + 1,
+				Template: vm.Name,
+				UserDataSecret: &corev1.LocalObjectReference{
+					Name: userDataSecretName,
+				},
+			},
+			cloneVM:     true,
+			machineName: "test0",
+		},
+		{
+			testCase: "fail on disc resize down",
+			providerSpec: vsphereapi.VSphereMachineProviderSpec{
+				CredentialsSecret: &corev1.LocalObjectReference{
+					Name: "test",
+				},
+				Workspace: &vsphereapi.Workspace{
+					Server: server.URL.Host,
+				},
+				DiskGiB:  defaultSizeGiB - 1,
+				Template: vm.Name,
+				UserDataSecret: &corev1.LocalObjectReference{
+					Name: userDataSecretName,
+				},
+			},
+			expectedError: errors.New("error getting disk spec for \"\": can't resize template disk down, initial capacity is larger: 5242880KiB > 4194304KiB"),
+		},
 		{
 			testCase: "fail on invalid resource pool",
 			providerSpec: vsphereapi.VSphereMachineProviderSpec{
@@ -151,6 +224,10 @@ func TestClone(t *testing.T) {
 					Server:       server.URL.Host,
 					ResourcePool: "invalid",
 				},
+				Template: vm.Name,
+				UserDataSecret: &corev1.LocalObjectReference{
+					Name: userDataSecretName,
+				},
 			},
 			expectedError: errors.New("resource pool not found, specify valid value"),
 		},
@@ -164,6 +241,10 @@ func TestClone(t *testing.T) {
 					Server:       server.URL.Host,
 					ResourcePool: "/DC0/host/DC0_C0/Resources/...",
 				},
+				Template: vm.Name,
+				UserDataSecret: &corev1.LocalObjectReference{
+					Name: userDataSecretName,
+				},
 			},
 			expectedError: errors.New("multiple resource pools found, specify one in config"),
 			setupFailureCondition: func() error {
@@ -195,6 +276,10 @@ func TestClone(t *testing.T) {
 					Server: server.URL.Host,
 					Folder: "invalid",
 				},
+				Template: vm.Name,
+				UserDataSecret: &corev1.LocalObjectReference{
+					Name: userDataSecretName,
+				},
 			},
 		},
 		{
@@ -207,6 +292,10 @@ func TestClone(t *testing.T) {
 					Server: server.URL.Host,
 					Folder: "/DC0/vm/...",
 				},
+				Template: vm.Name,
+				UserDataSecret: &corev1.LocalObjectReference{
+					Name: userDataSecretName,
+				},
 			},
 			expectedError: errors.New("multiple folders found, specify one in config"),
 			setupFailureCondition: func() error {
@@ -236,6 +325,10 @@ func TestClone(t *testing.T) {
 					Server:    server.URL.Host,
 					Datastore: "invalid",
 				},
+				Template: vm.Name,
+				UserDataSecret: &corev1.LocalObjectReference{
+					Name: userDataSecretName,
+				},
 			},
 			expectedError: errors.New("datastore not found, specify valid value"),
 		},
@@ -249,6 +342,10 @@ func TestClone(t *testing.T) {
 					Server:    server.URL.Host,
 					Datastore: "/DC0/...",
 				},
+				Template: vm.Name,
+				UserDataSecret: &corev1.LocalObjectReference{
+					Name: userDataSecretName,
+				},
 			},
 			expectedError: errors.New("multiple datastores found, specify one in config"),
 			setupFailureCondition: func() error {
@@ -276,6 +373,16 @@ func TestClone(t *testing.T) {
 				return nil
 			},
 		},
+		{
+			testCase: "fail on invalid template",
+			providerSpec: vsphereapi.VSphereMachineProviderSpec{
+				Template: "invalid",
+				UserDataSecret: &corev1.LocalObjectReference{
+					Name: userDataSecretName,
+				},
+			},
+			expectedError: errors.New("template not found, specify valid value"),
+		},
 	}
 
 	for _, tc := range testCases {
@@ -300,11 +407,9 @@ func TestClone(t *testing.T) {
 				providerSpec:   &tc.providerSpec,
 				session:        session,
 				providerStatus: &vsphereapi.VSphereMachineProviderStatus{},
-				client:         fake.NewFakeClientWithScheme(scheme.Scheme, &credentialsSecret),
+				client:         fake.NewFakeClientWithScheme(scheme.Scheme, &credentialsSecret, &userDataSecret),
 			}
 
-			machineScope.providerSpec.Template = vm.Name
-
 			if tc.machineName != "" {
 				machineScope.machine.Name = tc.machineName
 			}
@@ -455,9 +560,10 @@ func TestTaskIsFinished(t *testing.T) {
 			testCase: "task error is finished",
 			moTask: func() *mo.Task {
 				moTask.Info.State = types.TaskInfoStateError
+				moTask.Info.Error = &types.LocalizedMethodFault{}
 				return &moTask
 			},
-			expectError: false,
+			expectError: true,
 			finished:    true,
 		},
 		{
@@ -795,21 +901,34 @@ func TestReconcileTags(t *testing.T) {
 	testCases := []struct {
 		name          string
 		expectedError bool
+		attachTag     bool
+		testCondition func()
 	}{
 		{
-			name:          "Fail when tag doesn't exist",
-			expectedError: true,
+			name:          "Don't fail when tag doesn't exist",
+			expectedError: false,
 		},
 		{
 			name:          "Successfully attach a tag",
 			expectedError: false,
+			attachTag:     true,
+			testCondition: func() {
+				createTagAndCategory(session, "CLUSTERID_CATEGORY", tagName)
+			},
+		},
+		{
+			name:          "Fail on vSphere API error",
+			expectedError: false,
+			testCondition: func() {
+				session.Logout(context.Background())
+			},
 		},
 	}
 
 	for _, tc := range testCases {
 		t.Run(tc.name, func(t *testing.T) {
-			if !tc.expectedError {
-				createTagAndCategory(session, "CLUSTERID_CATEGORY", tagName)
+			if tc.testCondition != nil {
+				tc.testCondition()
 			}
 
 			err := vm.reconcileTags(context.TODO(), session, &machinev1.Machine{
@@ -827,21 +946,24 @@ func TestReconcileTags(t *testing.T) {
 				if err != nil {
 					t.Fatalf("Not expected error %v", err)
 				}
-				if err := session.WithRestClient(context.TODO(), func(c *rest.Client) error {
-					tagsMgr := tags.NewManager(c)
 
-					tags, err := tagsMgr.GetAttachedTags(context.TODO(), managedObjRef)
-					if err != nil {
-						return err
+				if tc.attachTag {
+					if err := session.WithRestClient(context.TODO(), func(c *rest.Client) error {
+						tagsMgr := tags.NewManager(c)
+
+						tags, err := tagsMgr.GetAttachedTags(context.TODO(), managedObjRef)
+						if err != nil {
+							return err
+						}
+
+						if tags[0].Name != tagName {
+							t.Fatalf("Expected tag %s, got %s", tagName, tags[0].Name)
+						}
+
+						return nil
+					}); err != nil {
+						t.Fatal(err)
 					}
-
-					if tags[0].Name != tagName {
-						t.Fatalf("Expected tag %s, got %s", tagName, tags[0].Name)
-					}
-
-					return nil
-				}); err != nil {
-					t.Fatal(err)
 				}
 			}
 
@@ -864,6 +986,7 @@ func TestCheckAttachedTag(t *testing.T) {
 	}
 
 	tagName := "CLUSTERID"
+	nonAttachedTagName := "nonAttachedTag"
 
 	if err := session.WithRestClient(context.TODO(), func(c *rest.Client) error {
 		tagsMgr := tags.NewManager(c)
@@ -889,6 +1012,14 @@ func TestCheckAttachedTag(t *testing.T) {
 			return err
 		}
 
+		_, err = tagsMgr.CreateTag(context.TODO(), &tags.Tag{
+			CategoryID: id,
+			Name:       nonAttachedTagName,
+		})
+		if err != nil {
+			return err
+		}
+
 		return nil
 	}); err != nil {
 		t.Fatal(err)
@@ -905,8 +1036,13 @@ func TestCheckAttachedTag(t *testing.T) {
 			tagName: tagName,
 		},
 		{
-			name:    "Fail to find a tag",
+			name:    "Return true if a tag doesn't exist",
 			tagName: "non existent tag",
+			findTag: true,
+		},
+		{
+			name:    "Fail to find a tag",
+			tagName: nonAttachedTagName,
 		},
 	}
 
@@ -1247,6 +1383,17 @@ func TestCreate(t *testing.T) {
 		},
 	}
 
+	userDataSecretName := "vsphere-ignition"
+	userDataSecret := &corev1.Secret{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      userDataSecretName,
+			Namespace: namespace,
+		},
+		Data: map[string][]byte{
+			userDataSecretKey: []byte("{}"),
+		},
+	}
+
 	cases := []struct {
 		name                  string
 		expectedError         error
@@ -1264,6 +1411,10 @@ func TestCreate(t *testing.T) {
 				CredentialsSecret: &corev1.LocalObjectReference{
 					Name: "test",
 				},
+				DiskGiB: 1,
+				UserDataSecret: &corev1.LocalObjectReference{
+					Name: userDataSecretName,
+				},
 			},
 		},
 		{
@@ -1315,7 +1466,8 @@ func TestCreate(t *testing.T) {
 			client := fake.NewFakeClientWithScheme(scheme.Scheme,
 				credentialsSecret,
 				configMap,
-				infra)
+				infra,
+				userDataSecret)
 
 			rawProviderSpec, err := vsphereapi.RawExtensionFromProviderSpec(&tc.providerSpec)
 			if err != nil {
